﻿using System.Diagnostics;
using System.Globalization;
using Azure;
using Azure.Core;
using Azure.Developer.LoadTesting;
using Azure.ResourceManager;
using Azure.ResourceManager.LoadTesting;
using Azure.ResourceManager.Resources;

namespace ALT.Shared;

public class ALTService
{
    private readonly TokenCredential credentials;

    public ALTService(TokenCredential credentials) => this.credentials = credentials;

    public async Task<ResourceGroupResource> GetResourceGroupAsync(string resourceGroupName,
        AzureLocation location)
    {
        var armClient = new ArmClient(credentials);
        var subscription = await armClient.GetDefaultSubscriptionAsync();
        var rgCollection = subscription.GetResourceGroups();
        ArmOperation<ResourceGroupResource> resourceGroupLro =
            await rgCollection.CreateOrUpdateAsync(WaitUntil.Completed, resourceGroupName,
                new ResourceGroupData(location));
        return resourceGroupLro.Value;
    }

    public async Task<ResourceGroupCollection> GetResourceGroupsAsync()
    {
        var armClient = new ArmClient(credentials);
        var subscription = await armClient.GetDefaultSubscriptionAsync();
        var rgCollection = subscription.GetResourceGroups();
        return rgCollection;
    }

    public async Task<LoadTestingResource> GetLoadTestingResourceAsync(ResourceGroupResource resourceGroup,
        string loadTestResourceName, AzureLocation location)
    {
        var loadTestingCollection = resourceGroup.GetLoadTestingResources();
        var inputPayload = new LoadTestingResourceData(location);
        ArmOperation<LoadTestingResource> loadTestingLro =
            await loadTestingCollection.CreateOrUpdateAsync(WaitUntil.Completed,
                loadTestResourceName, inputPayload);
        return loadTestingLro.Value;
    }

    public async Task<LoadTestingResourceCollection> GetLoadTestingResourcesAsync(string resourceGroupName)
    {
        var resourceGroup = await GetResourceGroupAsync(resourceGroupName, AzureLocation.WestEurope);
        var loadTestingCollection = resourceGroup.GetLoadTestingResources();
        return loadTestingCollection;
    }

    public async Task<LoadTestingResource> GetLoadTestingResourceAsync(string resourceGroupName)
    {
        var resourceGroup = await GetResourceGroupAsync(resourceGroupName, AzureLocation.WestEurope);
        var loadTestingCollection = resourceGroup.GetLoadTestingResources();
        return loadTestingCollection.FirstOrDefault();
    }

    public LoadTestAdministrationClient GetLoadTestAdminClient(Uri endpointUrl) =>
        new(endpointUrl, credentials);

    private object GetQuickTestConfiguration(string quickTestEndpointUrl, int virtualUsers = 1)
    {
        var data = new
        {
            description = $"This is created using SDK from {Environment.MachineName}",
            displayName = $"SDK's quick test from {Environment.MachineName}",
            loadTestConfiguration = new
            {
                engineInstances = 1,
                splitAllCSVs = false,
                quickStartTest = true,
                optionalLoadTestConfig = new
                {
                    endpointUrl = quickTestEndpointUrl,
                    virtualUsers,
                    rampUpTime = 0,
                    duration = 30,
                },
            },
            passFailCriteria = new
            {
                passFailMetrics = new
                {
                    condition1 = new
                    {
                        clientmetric = "error",
                        aggregate = "percentage",
                        condition = ">",
                        value = 50
                    }
                },
            }
        };

        return data;
    }

    public async Task<bool> CreateAndRunLoadTestAsync(string resourceGroupName, string loadTestResourceName,
        string urlToTest, int virtualUsers = 1)
    {
        var testName = StringExtensions.GenerateString(16);

        try
        {
            var resourceGroup = await GetResourceGroupAsync(resourceGroupName, AzureLocation.WestEurope);
            var loadTestingResource =
                await GetLoadTestingResourceAsync(resourceGroup, loadTestResourceName, AzureLocation.WestEurope);
            var loadTestAdminClient = GetLoadTestAdminClient(new Uri(loadTestingResource.Data.DataPlaneUri));
            GetQuickTest(loadTestAdminClient, testName, urlToTest, virtualUsers);
            RunLoadTest(new Uri(urlToTest), testName);
        }
        catch (Exception e)
        {
            Debug.WriteLine(e);
            return false;
        }

        return true;
    }

    public string GetQuickTest(LoadTestAdministrationClient loadTestAdministrationClient, string testId,
        string quickTestEndpointUrl, int virtualUsers = 1)
    {
        try
        {
            var response = loadTestAdministrationClient.CreateOrUpdateTest(testId,
                RequestContent.Create(GetQuickTestConfiguration(quickTestEndpointUrl, virtualUsers)));

            return response.Content.ToString();
        }
        catch (Exception ex)
        {
            Debug.WriteLine(ex);
            return ex.Message;
        }
    }

    public string RunLoadTest(Uri endpointUrl, string testId)
    {
        var loadTestRunClient = new LoadTestRunClient(endpointUrl, credentials);
        var testRunId = $"{Environment.MachineName}-run-{DateTime.Now.Ticks}".ToLower();

        var runData = new
        {
            testid = testId,
            displayName =
                $"Test run on {Environment.MachineName} at {DateTime.Now.ToString(CultureInfo.InvariantCulture)}"
        };

        try
        {
            var operation =
                loadTestRunClient.BeginTestRun(WaitUntil.Started, testRunId, RequestContent.Create(runData));

            // get inital response
            var initialResponse = operation.GetRawResponse();
            Debug.WriteLine(initialResponse);
            operation.WaitForCompletion();

            // final reponse
            var finalResponse = operation.GetRawResponse();
            return finalResponse.Content.ToString();
        }
        catch (Exception ex)
        {
            Debug.WriteLine(ex);
            return ex.Message;
        }
    }
    public async Task<string> CreateOrUpdateAppServiceComponentAsync(
        LoadTestAdministrationClient loadTestAdministrationClient,
        string resourceGroupName, string appServicePlan, string testId)
    {
        var armClient = new ArmClient(credentials);
        var subscription = await armClient.GetDefaultSubscriptionAsync();

        var planResourceId =
            $"/subscriptions/{subscription.Data.SubscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/serverfarms/{appServicePlan}";

        try
        {
            var response = loadTestAdministrationClient.CreateOrUpdateAppComponents(testId, RequestContent.Create(
                new Dictionary<string, Dictionary<string, Dictionary<string, string>>>
                {
                    {
                        "components", new Dictionary<string, Dictionary<string, string>>
                        {
                            {
                                planResourceId, new Dictionary<string, string>
                                {
                                    { "resourceId", planResourceId },
                                    { "resourceName", $"{appServicePlan}" },
                                    { "resourceType", "Microsoft.Web/serverfarms" },
                                    { "kind", "app" }
                                }
                            },
                        }
                    }
                }
            ));

            return response.Content.ToString();
        }
        catch (Exception ex)
        {
            Debug.WriteLine(ex);
            return ex.Message;
        }
    }
}