﻿using ALT.Shared;
using ALT.Web.Models;
using Azure.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;

namespace ALT.Web.Pages.Tests;

public class QuickCreatePageModel : PageModel
{
    private readonly ILogger<QuickCreatePageModel> logger;

    public QuickCreatePageModel(ILogger<QuickCreatePageModel> logger)
    {
        this.logger = logger;
    }

    public async Task OnGetAsync()
    {
        logger.LogInformation("Loading quick create page at {DateLoaded}", DateTime.Now);
        var altService = new ALTService(new AzurePowerShellCredential());
        var groupCollection = await altService.GetResourceGroupsAsync();
        logger.LogInformation("Getting {NumberOfResourceGroups} resource group in subscription",
            groupCollection.Count());
        var groups = await groupCollection.GetAllAsync().ToListAsync();
        foreach (var resourceGroupResource in groups)
        {
            ResourceGroups.Add(new ResourceGroupViewModel
            {
                Name = resourceGroupResource.Data.Name
            });
        }

        logger.LogInformation("Converted {ItemNumber} from api to local variables.", ResourceGroups.Count());
    }

    public async Task<IActionResult> OnPostAsync()
    {
        logger.LogInformation("Creating quick test");
        var form = await Request.ReadFormAsync();

        var resourceGroups = form["ddlResourceGroups"];
        var resourceGroup = resourceGroups[0];
        var urlToTest = form["UrlToTest"];
        logger.LogInformation("Url to test it {UrlToTest}", urlToTest);
        logger.LogInformation("Selected {RgName}", resourceGroup);
        var numberOfUsers = form["numberOfUsers"];
        logger.LogInformation("Number of virtual users {VirtualUserCount}", numberOfUsers);

        if (string.IsNullOrEmpty(urlToTest))
        {
            logger.LogWarning("Url is not entered, check form.");
            Message = "Url is not entered, check form.";
            return RedirectToPage("/Tests/QuickCreate");
        }

        try
        {
            var altService = new ALTService(new AzurePowerShellCredential());
            
            //load the first load testing resource
            var loadTestingResource = await altService.GetLoadTestingResourceAsync(resourceGroup);
            Message = "Load testing resource: " + loadTestingResource.Data.Name;
            logger.LogInformation("Load testing resource {LoadTestingResourceName}", loadTestingResource);
            await altService.CreateAndRunLoadTestAsync(resourceGroup, 
                loadTestingResource.Data.Name, 
                urlToTest,
                int.Parse(numberOfUsers));
            logger.LogInformation("Test is finished.");
        }
        catch (Exception e)
        {
            logger.LogError(e.Message);
            Message = $"There has been an error creating load test: {e.Message}";
        }

        Message = "Load test created, check https://portal.azure.com for more information";
        return RedirectToPage("/Tests/QuickCreate");
    }

    [BindProperty] public List<ResourceGroupViewModel> ResourceGroups { get; set; } = new();
    [TempData] [BindProperty] public string Message { get; set; }
}