﻿using System.Globalization;
using System.Text.RegularExpressions;
using Bogus;
using ITS.Models;
using ITS.SQL;
using Microsoft.Playwright;
using Microsoft.Playwright.NUnit;

namespace ITS.Tests;

[Parallelizable(ParallelScope.Fixtures)]
[TestFixture(Description = "Initial test to check if routes works as expected")]
public class Create_Task_Tests : PageTest
{
    private string urlToTest;
    private string pwd;
    private string sqlConnectionString;
    private ItsUser loggedInUser;

    [SetUp]
    public async Task Init()
    {
        urlToTest = Environment.GetEnvironmentVariable("WebUrl") ?? "https://localhost:5008";
        sqlConnectionString = Environment.GetEnvironmentVariable("SqlConnection") ??
                              @"Data Source=(LocalDb)\MSSQLLocalDB;Initial Catalog=ITSDB";
        var username = Environment.GetEnvironmentVariable("TestUsername") ?? "bojan@vrhovnik.net";
        pwd = Environment.GetEnvironmentVariable("TestPassword") ?? "ThisIsMyNTKPass!";
        var login = new ItsUserRepository(sqlConnectionString);
        loggedInUser = await login.LoginAsync(username, pwd);
    }

    [Test(Description = "Login and go to create page tasks")]
    public async Task GoToCreateWithLogin()
    {
        //login to system
        await using var browser = await Playwright.Chromium.LaunchAsync();
        var context = await browser.NewContextAsync();
        var page = await context.NewPageAsync();
        await page.GotoAsync($"{urlToTest}/user/login");
        await page.GetByLabel("Enter email").FillAsync(loggedInUser.Email);
        await page.GetByLabel("Enter Password").FillAsync(pwd);
        await page.GetByRole(AriaRole.Button, new() { Name = "Login me in" }).ClickAsync();
        //check if you have access
        await page.GotoAsync($"{urlToTest}/Tasks/Create");
        await Expect(page).ToHaveTitleAsync(new Regex("Add new task"));
    }

    [Test(Description = "Create page task")]
    public async Task CreateTaskAndCheckIfExistsInDatabase()
    {
        //login and get to the create task page
        await using var browser = await Playwright.Chromium.LaunchAsync();
        var context = await browser.NewContextAsync();
        var page = await context.NewPageAsync();
        await page.GotoAsync($"{urlToTest}/user/login");
        await page.GetByLabel("Enter email").FillAsync(loggedInUser.Email);
        await page.GetByLabel("Enter Password").FillAsync(pwd);
        await page.GetByRole(AriaRole.Button, new() { Name = "Login me in" }).ClickAsync();
        await page.GotoAsync($"{urlToTest}/Tasks/Create");
        //check, if you are on the right field
        await Expect(page).ToHaveTitleAsync(new Regex("Add new task"));

        var currentWorkTask = new Faker<WorkTask>()
            .RuleFor(workTask => workTask.Description,
                (faker, _) => faker.Lorem.Paragraph(new Random().Next(3, 10)))
            .RuleFor(u => u.Start, (f, _) => f.Date.Recent(new Random().Next(3, 40)))
            .RuleFor(u => u.End, (f, _) => f.Date.Future(new Random().Next(3, 20)))
            .RuleFor(u => u.IsPublic, (f, _) => f.Random.Bool())
            .RuleFor(u => u.IsCompleted, (f, _) => f.Random.Bool())
            .RuleFor(u => u.User, loggedInUser)
            .Generate();
        //add category
        var categoryRepository = new CategoryRepository(sqlConnectionString);
        var categories = await categoryRepository.GetAllAsync();
        Assert.That(categories, Is.Not.Null, "Categories are not null");
        Assert.That(categories.Count, Is.GreaterThan(0), "Categories has been pulled from database");
        var category = categories[new Random().Next(0, categories.Count - 1)];
        //fill the form
        await page.GetByLabel("Enter description").FillAsync(currentWorkTask.Description);
        await page.GetByLabel("Enter start date")
            .TypeAsync(currentWorkTask.Start
                .ToString(CultureInfo.InvariantCulture));
        await page.GetByLabel("Is publicly available").CheckAsync();
        await page.SelectOptionAsync("select#ddlCategories", category.CategoryId);
        //get tags
        var tagRepository = new TagRepository(sqlConnectionString);
        var tags = await tagRepository.GetAllAsync();
        Assert.That(tags, Is.Not.Null, "Tags are not null");
        Assert.That(tags.Count, Is.GreaterThan(0), "Tags has been pulled from database");
        var tagRangeToAdd = tags.GetRange(0, new Random().Next(1, tags.Count - 1))
            .Select(currentTag => currentTag.TagName);
        Assert.That(tagRangeToAdd.Count, Is.GreaterThan(0), "Count of tags to add must be more than 1");
        await page.SelectOptionAsync("select#ddlTags", tagRangeToAdd);
        await page.GetByRole(AriaRole.Button, new() { Name = "Save" }).ClickAsync();
        await Expect(page).ToHaveTitleAsync(new Regex($"Dashboard {loggedInUser.FullName}"));

        var workTaskRepository = new WorkTaskRepository(sqlConnectionString);
        currentWorkTask.Category = category;
        var tasksFromUser = await workTaskRepository.WorkTasksForUserAsync(loggedInUser.ItsUserId);
        //assert if item is available in database
        Assert.That(tasksFromUser.Count, Is.GreaterThan(0), "We have items in database for this user");
        var selectedTask = tasksFromUser.SingleOrDefault(cwt => cwt.IsCompleted == currentWorkTask.IsCompleted &&
                                                                cwt.Category.CategoryId == category.CategoryId &&
                                                                cwt.Description == currentWorkTask.Description &&
                                                                cwt.End == currentWorkTask.End);
        Assert.That(selectedTask, Is.Not.Null, $"Task with id {selectedTask} has been added to database");
    }
}