﻿using System.Text;
using Htmx;
using ITS.Core;
using ITS.Interfaces;
using ITS.Models;
using ITS.Web.Base;
using ITS.Web.Options;
using ITS.Web.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Caching.Distributed;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace ITS.Web.Pages.Tasks;

public class IndexRedisPageModel : BasePageModel
{
    private readonly ILogger<IndexRedisPageModel> logger;
    private readonly IWorkTaskRepository workTaskRepository;
    private readonly ReportApiHttpService reportApiHttpService;
    private readonly IUserDataContext userDataContext;
    private readonly IDistributedCache memoryCache;
    private AppOptions webOptions;
    private readonly ApiOptions apiOption;
    private RedisOptions redisOption;

    public IndexRedisPageModel(ILogger<IndexRedisPageModel> logger,
        IWorkTaskRepository workTaskRepository,
        IOptions<AppOptions> webSettingsValue,
        IOptions<ApiOptions> apiOptionsValue,
        IOptions<RedisOptions> redisOptionsValue,
        ReportApiHttpService reportApiHttpService,
        IUserDataContext userDataContext,
        IDistributedCache memoryCache)
    {
        this.logger = logger;
        webOptions = webSettingsValue.Value;
        apiOption = apiOptionsValue.Value;
        redisOption = redisOptionsValue.Value;
        this.workTaskRepository = workTaskRepository;
        this.reportApiHttpService = reportApiHttpService;
        this.userDataContext = userDataContext;
        this.memoryCache = memoryCache;
    }

    public async Task<IActionResult> OnGetAsync(int? pageNumber)
    {
        var currentPageNumber = pageNumber ?? 1;
        logger.LogInformation("Task search page loaded {DateStarted}", DateTime.Now);

        var tasks = await GetTaskFromCacheAsync("tasks");

        if (string.IsNullOrEmpty(Query)) Query = string.Empty;

        var currentTasks = tasks.Where(currentTask => currentTask.Description.Contains(Query)).ToList();
        WorkTasks = PaginatedList<WorkTask>.Create(currentTasks, currentPageNumber, webOptions.PageCount, Query);
        logger.LogInformation("Loaded {ItemCount} out of {AllCount} with {Query}. Generate Client URL.",
            WorkTasks.Count,
            WorkTasks.TotalPages, Query);

        ReportApiPublicTasks = $"{apiOption.ReportApiUrl}/tasks-api-reports/public-stats/pdf";
        logger.LogInformation("Url to download API is {ReportPublicTaskUrl}", ReportApiPublicTasks);

        try
        {
            logger.LogInformation("Calling HTTP service");
            var mostActiveTasks = await GetTaskFromCacheAsync("reports");
            var task = mostActiveTasks[0];
            MostActiveTask = $"Most active task is in {task.Category.Name} category";
            logger.LogInformation("Received {WorkTaskId} from http service", task.WorkTaskId);
        }
        catch (Exception e)
        {
            logger.LogError(e.Message);
        }

        if (!Request.IsHtmx()) return Page();

        //Response.Htmx(h => h.Push(Request.GetEncodedUrl()));//set item in browser url textbox
        return Partial("_WorkTasksList", WorkTasks);
    }

    private async Task<List<WorkTask>> GetTaskFromCacheAsync(string key)
    {
        var list = new List<WorkTask>();

        try
        {
            logger.LogInformation("Reading from distributed cache {RedisConnString}", redisOption.ConnectionString);
            var data = await memoryCache.GetAsync(key);
            var currentData = string.Empty;
            if (data == null)
            {
                logger.LogInformation("There is no data, load from database");
                list = await workTaskRepository.GetAsync();
                currentData = JsonConvert.SerializeObject(list);
                logger.LogInformation("Store {WorkTaskPayload} to Cache", currentData);
                var dataToStore = Encoding.UTF8.GetBytes(currentData);
                var options = new DistributedCacheEntryOptions()
                    .SetSlidingExpiration(TimeSpan.FromHours(1));
                await memoryCache.SetAsync(key, dataToStore, options);
                logger.LogInformation("Stored to distributed cache");
            }
            else
            {
                logger.LogInformation("Received {WorkTaskPayload} to get back to calling site", currentData);
                currentData = Encoding.UTF8.GetString(data);
            }

            if (string.IsNullOrEmpty(currentData))
                list = JsonConvert.DeserializeObject<List<WorkTask>>(currentData);
        }
        catch (Exception e)
        {
            logger.LogError(e.Message);
        }

        logger.LogInformation("Work task list count is {WorkTaskCount}", list.Count);
        return list;
    }

    [BindProperty(SupportsGet = true)] public string Query { get; set; }
    [BindProperty] public PaginatedList<WorkTask> WorkTasks { get; set; }
    [BindProperty] public string ReportApiPublicTasks { get; set; }
    [BindProperty] public string? MostActiveTask { get; set; }
}